@@ -33,8 +33,8 @@ def backup(verbose=False):
     timestamp = get_timestamp()
     create_directory(BACKUP_DIR)
     # Define the database names/paths
-    orig_db_path = os.path.join(INSTANCE_DIR, 'monopyly.sql')
-    backup_db_path = os.path.join(BACKUP_DIR, f'backup_{timestamp}.sql')
+    orig_db_path = os.path.join(INSTANCE_DIR, 'monopyly.sqlite')
+    backup_db_path = os.path.join(BACKUP_DIR, f'backup_{timestamp}.sqlite')
     # Connect to the databases
     db = sqlite3.connect(orig_db_path)
     backup_db = sqlite3.connect(backup_db_path)
